;****************************************************************************
;*  NAME:        r_cassini_mimi_chems
;*
;*  DESCRIPTION: Routine to read cassini_mimi_chems files. Adheres to
;*               PAPCO conventions: Error common block
;*                                  Uses MJDT time
;*                                  PAPCO read philosophy
;*
;*  INPUT:       none (uses common block time from PAPCO)
;*
;*  OUTPUT:      none (places data into common blocks)
;*
;* MODIFICATION HISTORY:       
;*     written April 2001, S. BOURDARIE (ONERA/DESP)
;   3/18/2002 M.B. Kusterer Changed the routine to read the one day files. Stopped hourly loop
;   Sept. 4 2001, M.B.Kusterer  Added call to papco_get_remote_data to r_cassini_mimi_chems
;****************************************************************************
pro r_cassini_mimi_chems, VERBOSE=VERBOSE

   common cassini_mimi_chems_data, input_header, input_data
   common mjdt, mjdt_start, mjdt_end
   common get_error, get_err_no, get_err_msg

; common get_error is used by PAPCO to check if the data read was successful.
; it is up to the user to set this correctly and to make sure that the read- 
; routine does not crash on an I/O error - otherwise PAPCO will crash at that
; point, too. PAPCO draws an empty panel when get_err_no in not equal to zero
; and prints get_err_msg in that panel for user information. Papco will try
; to invoce the plot routine for this data if get_err_no = 0 , otherwise an
; empty panel with the error message get_err_msg is plotted.

   get_err_no=0
   get_err_msg='cassini_mimi_chems read succesfully'

; check for env. variable pointing to data directory
   data_dir='CASSINI_MIMI_CHEMS_DATA'
   c_path=getenv(data_dir)
   if strlen(c_path) eq 0 then begin
      get_err_no=1
      get_err_msg='Environmental variable '+data_dir+' not set'
      print,get_err_msg
      return
   endif
   
; check for existence of data directory
   time=mjdt_start 
   convert_t90_to_date, time, year, day_oy, hour, minute, second,$
                          month, cmonth, day_om, /mjdt 
   yy = string(year,format='(i4.4)')
   res=findfile(c_path+'/'+yy+'/',count=cc)
   if cc eq 0 then begin
      get_err_no=2
      get_err_msg='Data directory '+c_path+' not found'
      print,get_err_msg
     return
   endif
   

; check VERBOSE keyword and set
   if keyword_set(VERBOSE) then verbose=VERBOSE else verbose=0     
   
; clear data arrays for reuse
   input_header=0  &  input_data=0   
   
; loop through from start to end time concatonating data files into one
; array. Here we assume that the data is stored in files per day, and
; that the time info in each file is seconds of day. papco standard is
; to hand over data to the plot routines with the time running in
; seconds from the number of seconds of the startday.
   
   time=mjdt_start  &  no_files_read=0  &  day_count=0.0
        ;MBK  
   oldfile = ''
        ;MBK  
   
   while mjdt_end.mjd ge time.mjd do begin
   
        
     if mjdt_end.mjd eq time.mjd then hour_end = mjdt_end.t else hour_end=86400.
     while time.t le hour_end do begin
     
     
        
        ; assemble the data file from the time range requested
        convert_t90_to_date, time, year, day_oy, hour, minute, second,$
                          month, cmonth, day_om, /mjdt   
        ;MBK
        fln = '/'+string(year,format='(i4.4)')+'/cmichs_1A_' $
                 +string(year,day_oy,format='(i4.4,i3.3)')
        newfile = fln
        
        IF newfile NE oldfile THEN BEGIN
        ;MBK
        
            found=findfile(c_path+fln+'*.*',count=nfs);are files there *.idl or *.txt or .txt.gz?

            if nfs ne 0 then begin
              ; read one hour of data
	          resul = str_sep(found(0),'.')
              r_cassini_mimi_chems_onefile, resul(0), /VERBOSE


              ;if a file was succesfully read, concatonate!
              if get_err_no eq 0 then begin 
                if no_files_read eq 0 then begin   ;start the array buffer
                  input_data_buf=temporary(input_data)
                  input_header_buf = input_header
                endif else begin                   ;otherwise concatonate arrays
                  input_data_buf=[temporary(input_data_buf),temporary(input_data)]
                endelse        
                input_data=0                     ;clear array for reuse
                no_files_read=no_files_read+1
              endif
	        endif else begin    ;didn't fine any files to match, try ftp from mimi
                    tempname = c_path+fln
                    print,'% failure to find: '+tempname+' trying ftp next ...'

                    ;make remote path fln is local path/file with no extension
                    ll = STRLEN(tempname)
                    year = STRMID(tempname,ll-7,4)       
                    file = STRMID(tempname,ll-17,26)      ;filename with no path
                    ldir = STRMID(tempname,0,ll-17)    ;path to place file at locally
                    rdir = 'papdata_v35/chemssci/'+year+'/' ;remote directory to find file

                    ;call get remote data
                    papco_get_remote_data,'ftp://mimipap:MIMIpap1@sd-ftp.jhuapl.edu//'+rdir,rdir+file+'*.idl',ldir

                    found=findfile(c_path+fln+'*.*',count=nfs)
                    if nfs ne 0 then begin
                        resul = str_sep(found(0),'.')

                        r_cassini_mimi_chems_onefile, resul(0), /VERBOSE
                        ;if a file was succesfully read, concatonate!
                        if get_err_no eq 0 then begin 
                            if no_files_read eq 0 then begin   ;start the array buffer
                                input_data_buf=temporary(input_data)
                                input_header_buf = input_header
                            endif else begin                   ;otherwise concatonate arrays
                                input_data_buf=[temporary(input_data_buf),temporary(input_data)]
                            endelse        
                            input_data=0                     ;clear array for reuse
                            no_files_read=no_files_read+1
                        endif
                    endif   ;2nd try at nfs ne 0


                endelse     ;nfs ne 0
       ;MBK
        oldfile = newfile
       ENDIF 
       ;MBK
	time.t = time.t+(3600.*24.)
      endwhile
      
      
      time.t = 0.
      time.mjd=time.mjd+1
   endwhile         
   
; check if any files were read   
   if (no_files_read ne 0) and (get_err_no ne 1) then begin
     get_err_no=0
     get_err_msg='% r_cassini_mimi_chems: '+string(no_files_read,format="(i2)")+ $
       ' files read succesfully'
     ;clear arrays and load buffer into output array
     input_data=0  &  input_data=input_data_buf  &  input_data_buf=0
     input_header = input_header_buf

;converting time to PAPCO plot standard
     input_data(*).time=(input_data(*).mjdt_time.mjd-mjdt_start.mjd) $
                     *86400+input_data(*).mjdt_time.t
   endif else begin
     get_err_no=1
     get_err_msg='% r_cassini_mimi_chems: no files found for timerange'
     return
   endelse      
   
; restrict data to actual time range requested
   ut1=mjdt_start.t
   ut2=long(mjdt_end.mjd-mjdt_start.mjd)*86400l+mjdt_end.t
   index=where((input_data.Time ge ut1) and (input_data.Time le ut2),count)
   if count ne 0 then begin
      input_data=input_data(index)
   endif else begin
     get_err_no=1
     get_err_msg='% r_cassini_chems: no data found for timerange'
     return
   endelse      
  
end

;******************************************************************************
;*  NAME:        r_cassini_mimi_chems_onefile,fln
;*
;*  DESCRIPTION: reads one file of type plot_file. Assumes data has been
;*               stored in IDL saveset using structures input_header and
;*               input_data
;*
;*  INPUT:       fln  - string of filename
;*
;*  OUTPUT:      none (places data into common blocks)
;*
;* MODIFICATION HISTORY:       
;*     written April 2001, S. BOURDARIE (ONERA/DESP)
;******************************************************************************
pro r_cassini_mimi_chems_onefile, fln, VERBOSE=VERBOSE
  
   common cassini_mimi_chems_data, input_header, input_data
   common get_error, get_err_no, get_err_msg
   
   get_err_no=0
   get_err_msg='cassini_mimi_chems onefile read succesfully'
   
; clear data arrays for reuse
   input_header=0  &  input_data=0   
   
   on_ioerror, read_original
   
   restore,fln+'.idl',/verbose
   dealloc_lun,/quiet    ;frees logical units used by save/restore
   print,'% r_cassini_mimi_chems_onefile: idl saveset read succesfully'
   return

   read_original:
   ; if the .idl save file could not be found, then try to read the original
   r_cassini_mimi_chems_oneday_original, fln
      
end   

;****************************************************************************
;*  NAME:        getmonth_onefile
;*
;*  DESCRIPTION: Routine to read cassini_mimi_chems files. Adheres to
;*               PAPCO conventions: Error common block
;*                                  Uses MJDT time
;*                                  PAPCO read philosophy
;*
;*  INPUT:       none (uses common block time from PAPCO)
;*
;*  OUTPUT:      none (places data into common blocks)
;*
;* MODIFICATION HISTORY:       
;*     written April 2001, S. BOURDARIE (ONERA/DESP)
;****************************************************************************
pro   r_cassini_mimi_chems_oneday_original, fln

   common cassini_mimi_chems_data, input_header, input_data
   common get_error, get_err_no, get_err_msg

;  clear exisiting data array
   input_data=0

   print,'% r_cassini_mimi_chems_ascii: Reading ',fln+'.txt',' (slow)'

   time={MJDT, MJD:0l, T:0.0}
   dummystr=''
   dummytime=' 0 0 0'
   dummy_struct={    data      :lon64arr(3,32), $
                     mjdt_time :time, $
                     time      :0.0d0}
 
   input_data=replicate(dummy_struct,90000)
   input_data(*).data(0,*)=196602ll
   input_data(*).data(1:2,*)=65535ll
   openr,f_Unit,fln+'.txt', /GET_LUN, ERROR=err

   IF err NE 0 THEN BEGIN
      openr,f_Unit,fln+'.txt.gz',/GET_LUN,/compress, ERROR=err
      IF err ne 0 THEN BEGIN
         print, '% read cassini_mimi_chems: '+fln+'.txt'+'  :  not available.'
         get_err_no=2
         get_err_msg=!ERR_STRING
         print, !ERR_STRING
         RETURN
       ENDIF
   ENDIF 

; read the header
   burp = ''
   readf,f_Unit,burp
   while (strmid(burp,0,12) ne 'Spin_Counter') do readf,f_Unit,burp

   strdata=''
   data = lon64arr(33)
   i=-1
   flag=0
   while not eof(f_Unit) do begin 
      readf,f_Unit,strdata
      str = strmid(strdata,242,strlen(strdata)-242)
      reads,str,data
      if data(0) eq 1 and flag eq 1 then i=i+1
      if data(0) eq 31 and flag eq 0 then begin
         flag=1
	 i=i+1
      endif
      if flag eq 1 and data(0) ne 65535ll AND (data[0] GE 0 AND data[0] LE 31) then begin
         year = fix(strmid(strdata,176,4))
         doy  = fix(strmid(strdata,181,3))
         hour = fix(strmid(strdata,185,2))
         minutes = fix(strmid(strdata,188,2))
         seconds = float(strmid(strdata,191,6))
         input_data(i).mjdt_time=convert_date_to_t90(doy=doy, year=year,$
                               hour=hour,minutes=minutes,seconds=seconds, /MJDT)
                               

         input_data(i).data(0,data(0))  = data(4)+data(16)<196602ll
         input_data(i).data(1,data(0))  = data(29)+data(30)<65535ll
         input_data(i).data(2,data(0))  = data(31)+data(32)<65535ll
      endif
   endwhile

   close,f_Unit
   free_lun,f_Unit

   IF i le 0 THEN BEGIN
      print, '% readcassini_mimi_chems: '+fln+'.txt'+'  :  not available.'
      get_err_no=2
      get_err_msg=!ERR_STRING
      print, !ERR_STRING
      RETURN
   ENDIF

   input_data=temporary(input_data(0:i))

; save read data with correct time. Rename data to *.idl
  save,filename=fln+'.idl',input_data,/verbose

END
