;******************************************************************************
;* PROCEDURE:     
;*      cassini_mimi_lemms_light_removal,input_data
;* 
;* DESCRIPTION:  
;*	The light sensitivity of LEMMS is a significant problem.  
;*      This procedure gives an algorithm for removing much of this light 
;*      contamination    
;*
;* INPUTS:       
;*	input_data: data to be cleaned up
;*                  input_data contains selected data points
;*                  (within selected time range and selected
;*                   subsector range).
;*              
;* OUTPUTS: 
;*   	input_data after cleaning
;*;
;*  Notes:      Lemms priority channels 
;               before 1999-229T11:28 channels A0 A1 E0 E1
;               after 1999-229T11:28 channels A0 A1 C0 C1
;               after 2003-255 channels A1 A5 C1 C5 E4 P2
;* KEYWORDS: NONE
;* 
;* CALLING SEQUENCE:
;*       cassini_mimi_lemms_light_removal,input_data
;*
;* MODIFICATION HISTORY: 
;*     written April 2001, Andreas Lagg (MPAe)     
;*     Modified April 2001, S. BOURDARIE (ONERA/DESP) (for PAPCO LEMMS module)
;   April 2003, M.B. Kusterer (JHU/APL) Finished adding priority channels, changed to just
;                           store the min/max of A0,C0 in input_data to speed up call.
;   Mar 1 2004, M.B.Kusterer  Changed to work with the changed and 2 additional priority channels.
;               For data before 1999-229 use A0, A1, for data after 2003-255 use A1 C1 
;******************************************************************************
pro cassini_mimi_lemms_light_removal,input_data, input_data_prior

   common cassini_mimi_lemms_description,emin,emax,pmin,pmax

   print,'% CASSINI_MIMI_LEMMS_LIGHT_REMOVAL: Light removal'
   chmimi=define_cassini_mimi_lemms_channels()

    lev_a=10                    ;bg threshold
    lev_c=10
    maxlev_a=5
    maxlev_c=4
    mxgam=0.
    
    nit=n_elements(input_data)  ;input_data contains selected data points
                                ;(within selected time range and selected
                                ;subsector range).
    
    
                                ;first check: ratio between min and
                                ;max of counts in 8 microsectors of A0
                                ;MBK = changed to find matching priority A 0 channel
                                ;using time and then following 7 will be all microsectors
    lc_a0_mm=fltarr(nit,2)
    ;for imm=0l,nit-1l do $
     ; lc_a0_mm(imm,*)=[min(input_data(imm).PrioA0(*)), $
             ;          max(input_data(imm).PrioA0(*))]
                       
     for imm=0l,nit-1l do $
      lc_a0_mm(imm,*)=[input_data(imm).A0_minmax(0), input_data(imm).A0_minmax(1)]
              
                       
    a0_ms_ratio=lc_a0_mm(*,1)/(lc_a0_mm(*,0)>1)
    
    
                                ;second check: ratio between min and
                                ;max of counts in 8 microsectors of C0
                                ;MBK = changed to find matching priority C 0 channel
                                ;using time and then following 7 will be all microsectors
    lc_c0_mm=fltarr(nit,2)
    ;for imm=0l,nit-1l do $
     ; lc_c0_mm(imm,*)=[min(input_data(imm).PrioC0(*)) $
                   ;    ,max(input_data(imm).PrioC0(*))]
                   
    for imm=0l,nit-1l do $
      lc_c0_mm(imm,*)=[input_data(imm).C0_minmax(0) ,input_data(imm).C0_minmax(1)]
     
    
    c0_ms_ratio=lc_c0_mm(*,1)/(lc_c0_mm(*,0)>1)
    
                                ;read channels A0, A1, A2, A3. Set
                                ;'background' threshold, do not allow
                                ;counts less than bg threshold

    lc_a0=input_data.data(0)>lev_a
    lc_a1=input_data.data(1)>lev_a
    lc_a2=input_data.data(2)>lev_a
    lc_a3=input_data.data(3)>lev_a
    
                                ;calculate spectral indices (no real
                                ;conversion to flux, simply use
                                ;counts, because geometric factors and
                                ;sector time cancel out when building
                                ;the ratio). pmin and pmax are
                                ;energies of corresponding channel
    g01_a=(alog((lc_a0)/(chmimi(0).e_hi-chmimi(0).e_lo)/ $
                (lc_a1/(chmimi(1).e_hi-chmimi(1).e_lo)))/ $
           alog(chmimi(0).e_lo/chmimi(1).e_lo))
    g23_a=(alog((lc_a2/(chmimi(2).e_hi-chmimi(2).e_lo))/ $
                (lc_a3/(chmimi(3).e_hi-chmimi(3).e_lo)))/ $
           alog(chmimi(2).e_lo/chmimi(3).e_lo))
    
                                ;same for electron channels C0, C1,
                                ;C2, C3
                                ;gxx_c is no spectral index because geometric
                                ;factors are not the same for C0, C1,
                                ;C2, C3! 
    lc_c0=input_data.data(14)>lev_c
    lc_c1=input_data.data(15)>lev_c
    lc_c2=input_data.data(16)>lev_c
    lc_c3=input_data.data(17)>lev_c

    g01_c=(alog((lc_c0/(chmimi(14).e_hi-chmimi(14).e_lo))/ $
                (lc_c1/(chmimi(15).e_hi-chmimi(15).e_lo)))/ $
           alog(chmimi(14).e_lo/chmimi(15).e_lo))
    g23_c=(alog((lc_c2/(chmimi(16).e_hi-chmimi(16).e_lo))/ $
                (lc_c3/(chmimi(17).e_hi-chmimi(17).e_lo)))/ $
           alog(chmimi(16).e_lo/chmimi(17).e_lo))
    
                                ;condition for light in detector A
                                ;1. spec index >= -12
                                ;2. microsector ratio has to be <= 100
    light_idx_a=where((g01_a lt -12) or (g23_a gt 0.5) or $
                      a0_ms_ratio gt 1e2)
    
                                ;condition for light in detector C
                                ;1. 'spec index' >= -1.5
                                ;2. microsector ratio has to be <= 100
    light_idx_c=where((g01_c lt -1.5) or (g23_c lt -5) or $
                      c0_ms_ratio gt 1e2)
    
                                ;select affected channels: all A
                                ;channels for light_idx_a
  if light_idx_a(0) ne -1 then $
    input_data(light_idx_a).data(0:8)=4294967295ll
    
                                ;select affected channels: all C
                                ;channels for light_idx_c
  if light_idx_c(0) ne -1 then $
    input_data(light_idx_c).data(14:21)=4294967295ll
    
    
  if light_idx_a(0) ne -1 then begin
                                ;select affected priority channels: all A
                                ;channels for light_idx_a
    cnt = N_ELEMENTS(input_data)
    
    input_data_prior = REFORM(input_data_prior,8,cnt)
    input_data_prior[*,light_idx_a].data(0:1) = 4294967295ll
    input_data_prior = REFORM(input_data_prior,8*cnt)
  endif
    
    
                                ;select affected priority channels: all C
                                ;channels for light_idx_c, 8 is every other microsector value
  if light_idx_c(0) ne -1 then begin
    cnt = N_ELEMENTS(input_data)
    input_data_prior = REFORM(input_data_prior,8,cnt)
    input_data_prior[*,light_idx_c].data(2:3) = 4294967295ll
    input_data_prior = REFORM(input_data_prior,8*cnt)
  endif

end
