;******************************************************************************
;* file p_cassini_mimi_lemms_subsector.pro
;*
;* this file is a template for a plot routine which can be used by papco.
;* it includes all the periphery needed, the user simply has to add in
;* the actual plot statement for his/her kind of data.
;*
;* This is a working example for a pre-defined example data array.
;******************************************************************************
;******************************************************************************
;* PROCEDURE:     
;*      p_cassini_mimi_lemms_subsector, panel, type, $
;*           OVERPLOT=OVERPLOT,$
;*           PLOTS_ATTHISPOSITION=PLOTS_ATTHISPOSITION,$
;*           SUBTABLE=SUBTABLE, $
;*           _EXTRA=extra_par
;* 
;* DESCRIPTION:  
;*	This procedure plots data of type data_type for inclusion in papco.  
;*      Data must have been read and placed into common blocks before this
;*      routine is called. Behaviour is controlled by the vector panel
;*	which controls positioning of the plot (see papco_conventions.text
;*      in $papco). In general, the following plot conventions are used:
;*      	- if the panel is at the bottom plot time axis plus any
;*                additional axis (ie ephemeris) needed.
;*              - if panel is at the top also plot a title
;*              - a descriptor for panels is plotted to the right of each
;*                panel, rotated by 90 deg if there is a papco_colorbar
;*              - panels all have common time
;*              - y-scaling (or z-scaling for color plots) is either
;*              - automatic or manually set using common yscale or zscale    
;*
;* INPUTS:       
;*	panel	three element integer vector. Controls positioning of the plot
;*              (see papco_conventions.text in $papco)
;*      type    optional parameter for options. Normally is a four element
;*              integer vector. This is normally equivalent to the field
;*              typeVector of the papco structure papco_PLOTINFO which is set
;*              by the panel editor. Additional fields that can be used if
;*              needed are papco_PLOTINFO.switch (three element vector) and 
;*              papco_PLOTINFO.channel (integer). If your plot routine needs
;*              more input fields than these you have to extend the definition
;*              of the structure papco_PLOTINFO
;*              (procedure papco_getPlotInfoStruct in $papco/papco.pro). Try to
;*              avoid this!
;*              
;* OUTPUTS: 
;*   	none
;*
;* KEYWORDS:
;*      OVERPLOT  if you support this you can alter the appearance of your
;*              plot if it is an overplot to an exisiting panel. Standard is
;*              to plot data, no axis, using the y-scaling of the first plot
;*              and to shift the labeling on the left down one line. Useful
;*              to plot model data together with real data, ie mag. field
;*              (see p_crres_mag for an example).
;*      PLOTS_ATTHISPOSITION  used to control the left side labeling for more
;*              than one overplot at the same position.
;*      _EXTRA  the plot structure containing some keyword for calls to
;*              plot. Used to customize color and line style.   
;* 
;* CALLING SEQUENCE:
;*       p_map, panel, type, $
;*           OVERPLOT=OVERPLOT, PLOTS_ATTHISPOSITION=PLOTS_ATTHISPOSITION,$
;*           _EXTRA=extra_par
;*
;* MODIFICATION HISTORY:       
;*     written April 2001, S. BOURDARIE (ONERA/DESP)
;   April 2003, M.B. Kusterer (JHU/APL) Finished adding priority channels, reorganized into 
;                adding one more structure input_data_prior,to the data common block. It 
;                seemed the most memory efficient way due to the time difference for the priority.
;               Also added code to figure out based on time which priority channels are available.
;               I do store the min,max of A0,C0 in input_data now to speed up light removal call.
;               I had to make an if statement to separate the priority data from the normal.
;               Also changed all loop indexes to be longs in dealing with data arrays.
;   Apr 2003, M.B.Kusterer (JHU/APL) Added count rates and counts as a plotting option, stored in type(1).
;   Mar 1 2004, M.B.Kusterer  Changed to work with the changed and 2 additional priority channels. 
;               Light correction only works for dates  1999-229 to 2003-255
;******************************************************************************
pro p_cassini_mimi_lemms_subsector, panel,type, channel,OUTPUT=OUTPUT, $
                      OVERPLOT=OVERPLOT, $
                      PLOTS_ATTHISPOSITION=PLOTS_ATTHISPOSITION, $
                      SUBTABLE=SUBTABLE, $
                      _EXTRA=extra_par

; It doesn't make much sense to use a color plot as an overplot, as it will
; hide what's below. Line styles are also not useful. So the keywords 
; OVERPLOT=OVERPLOT, PLOTS_ATTHISPOSITION=PLOTS_ATTHISPOSITION,_EXTRA=extra_par
; are not used at all here, but are in the call for consistency...

; NOTE: I use a little trick in passing in the keywords to to this
; routine. The calling routine's call is simply:
;         p_map_line, panelVector, currPlotInfo.typeVector(0), $ 
;                           _EXTRA=extra_PlotInfo
; extra_PlotInfo contains the tags OVERPLOT, PLOTS_ATTHISPOSITION,
; SUBTABLE and then some tags which are meant to be passed to the plot
; call. By putting in the explicit keywords in the procedure definition they
; get split out of the structure extra_PlotInfo that was passed in, and only
; those tags whichare meant for the plot routine remain in extra_par.

; Comments here should include an explanation of the plot options used
; in type, switch. Replace type in the call with whatever variables
; you need, or add more

; Comments here should include an explanation of the plot options used
; in type, switch. Replace type in the call with whatever variables
; you need, or add more

; The following common blocks are needed:

   common mjdt, mjdt_start, mjdt_end    ;common time limit in mjdt
   common yscale, yscl			;man/auto yscaling
   common zscale, zscl			;man/auto zscaling
   common shift_label, down             ;common for x-axis label shifting
   common coordinateSystems, plotted_x, plotted_y  
                                        ;info on coords used by mouse

; One of the common blocks should contain the plot data as returned by the
; data read procedure r_cassini_mimi_lemms. 

   common cassini_mimi_lemms_data, input_header, input_data, input_data_prior
   common time, xut1,xut2

   papco_Set_SubTable, SUBTABLE         ;sets the papco color table index
   panelset,panel			;sets the panel position viewport

; option to automatically calculate ytick numbers in order to avoid to
; many ticks if panels get too narrow. Maximum is set to 10. You may
; leave this out and allow plot to use the built-in defaults.

   ytickno=50*(!p.position(3)-!p.position(1))
   if ytickno gt 10 then ytickno=10

; since there is a common time axis plots look better if they all use the same
; number of xticks. In papco, this number is 6 by default. This is set by papco
; using the IDL graphics system variable !X.TICKS.
; You can overide this for your own plot by setting the varbale
; below. However, for plotting panels together, it makes sense to stick to the
; default. If you do set this variable, it needs to be included in the 
; extra_plotPar_common definition below.

;   xtickno=6

; NB!!! NB!!! NB!!! NB!!! NB!!! NB!!! NB!!! NB!!! 
; Avoid setting the global IDL graphics system variables in your plot routine,
; as they may affect other modules' plot routines. The IDL graphics system 
; variables are set by papco to default values that gurantee some uniformity
; of plot appearance and should not be tampered with - unless you don't
; care if your plot "looks good" when combined with the other modules.
; NB!!! NB!!! NB!!! NB!!! NB!!! NB!!! NB!!! NB!!! 

; for the color plot routine, you need to set up arrays containing the upper  
; and lower limits of each channel. Here is an example for CRRES MEB energy 
; channels. There must be as many entries as there are channels in the data
; array! 

  yminarr=findgen(16)
  ymaxarr=findgen(16)+1.
  n_channels=n_elements(yminarr)
  yarr=fltarr(n_channels,2)   
  yarr(*,0)= yminarr 
  yarr(*,1)=ymaxarr

; for color plots, the y-axis normally refers to a channel. You have to 
; re-scale the data array to the range required before passing it to the
; color plot routine, also the ymin and ymax arrays.

; the plot-calls normally use the arrays time for the time axis and yarr (ymat 
; for color plots) for the y or z axis.
; depending on your plot options different data may be selected and different
; axis-labeling may be needed. This is normally done using a case statement.

; in this example we have a single test array, so no case statement is needed.

; set the begin end end times. papco convention is to use running secconds
; starting with the no of seconds since the begiing of the start day.
   time=input_data.time

; restrict data to actual time range requested - this makes for faster zooming
   xut1=mjdt_start.t                       
   xut2=long(mjdt_end.mjd-mjdt_start.mjd)*86400l+mjdt_end.t

   index=where((time ge xut1) and (time le xut2),count)
   if count ne 0 then begin
      input_data=input_data(index)
      time=time(index)
   endif
   
;Added same function for priority channels, time between two points (needed not to plot missing data), MBK
   time_prior=input_data_prior.time
   if count ne 0 then begin
      index_prior = index[0]*8 + L64INDGEN(count*8)
      time_prior=time_prior(index_prior)
      input_data_prior=input_data_prior(index_prior)
   endif

   typeNames=['A0','A1','A2','A3','A4','A5','A6','A7','A8', $
              'B0','B1','B2','B3','BE', $
              'C0','C1','C2','C3','C4','C5','C6','C7', $
	      'AS','BS','ES','E2SLF2S','FS', $
              'E0','E1','E2','E3','E4','E5','E6','E7', $
	      'G1',$
              'P1','P2','P3','P4','P5','P6','P7','P8','P9', $
              'H1','H2','H3','H4','H5','Z1','Z2','Z3', $
              'D1','D2','D3','D41','Blk','Blk','Blk',$
              'Prio_A0orA1','Prio_A1orA5','Prio_E0orC0orC1','Prio_E1orC1orC5','Prio_E4','Prio_P2']


   ;
   ; Convert into count rates or flux if selected
   ;
   if type(1) NE 0 THEN BEGIN
        cassini_mimi_lemms_convert,type(1),input_data,input_data_prior
   ENDIF
;
       
    ;figure out priority channel name based on time
    cassini_mimi_lemms_priority, mjdt_start, currprior, numprior, currindex 
; Process normal channels
;
    IF channel LT 60 THEN BEGIN  


       yray=fltarr(count,n_elements(yminarr))
       tps=dblarr(count)
        IF type(1) EQ 0 THEN uztit='[counts]' $
        ELSE uztit='[counts/acc]'
       uytitle='subsector'
       utitle='CASSINI-MIMI/LEMMS!c'+typeNames(channel)

       if type(0) eq 0 then utitle=utitle+' not av.'
       if type(0) eq 1 then utitle=utitle+' spin av.'

       nodat=4294967295ll

    ; remove light contamination
       if (channel le 8) OR (channel ge 14 and channel le 21) OR (channel ge 60) then begin
       
           IF currindex EQ 2 THEN BEGIN
                print,'% CASSINI_MIMI_LEMMS_LIGHT_REMOVAL: No Light removal defined for this time period'
           ENDIF ELSE IF currindex EQ 0 THEN BEGIN
                print,'% CASSINI_MIMI_LEMMS_LIGHT_REMOVAL: No Light removal defined for this time period'
           ENDIF ELSE BEGIN

                cassini_mimi_lemms_light_removal,input_data, input_data_prior
           ENDELSE
        endif

       for i=0,15 do begin
          indix=where(input_data.subsector eq i, count)
          input_data_buf=input_data(indix)
          if type(0) eq 0 then begin  ;no average
             yray(0:count-1,i)=input_data_buf(*).data(channel)
          endif

          if type(0) eq 1 then begin  ;spin average
	     if input_data_buf(0).data(channel) ne nodat then begin
                yray(0,i)=input_data_buf(0).data(channel)
       	        np=1
	     endif else begin
	        yray(0,i)=0.
	        np=0
	     endelse
	     nav = 0
	     spin=input_data_buf(0).spin
             for j=1L,n_elements(input_data_buf)-1 do begin
	        if input_data_buf(j).spin eq spin then begin
	          if input_data_buf(j).data(channel) ne nodat then begin
	             yray(nav,i)=yray(nav,i)+input_data_buf(j).data(channel)
                     np=np+1
	          endif
                endif else begin
	          if np ne 0 then begin
                     yray(nav,i)=yray(nav,i)/float(np)
	          endif else begin
	             yray(nav,i)=nodat
	          endelse
	          nav=nav+1
	          if input_data_buf(j).data(channel) ne nodat then begin
                     yray(nav,i)=input_data_buf(j).data(channel)
       	             np=1
	          endif else begin
	             yray(nav,i)=0.
	             np=0
	          endelse
	          spin=input_data_buf(j).spin
                endelse
	     endfor
	     if np ne 0 then begin
                yray(nav,i)=yray(nav,i)/float(np)
	     endif else begin
	        yray(nav,i)=nodat
	     endelse
          endif
       endfor

       if type(0) eq 0 then begin  ;average time over subsectors
          tps(0)=time(0)
          np=1
          nav = 0
          sect=input_data(0).sector
          for j=1L,n_elements(time)-1 do begin
	      if input_data(j).sector eq sect then begin
                 tps(nav)=tps(nav)+time(j)
                 np=np+1
              endif else begin
	         if np ne 0 then tps(nav)=tps(nav)/float(np) $
	         else tps(nav)=0.d0
	         nav=nav+1
                 tps(nav)=time(j)
       	         np=1
	         sect=input_data(j).sector
              endelse
          endfor
          if np ne 0 then tps(nav)=tps(nav)/float(np) $
          else tps(nav)=0.d0
       endif

       if type(0) eq 1 then begin  ;average time over spin
          tps(0)=time(0)
          np=1
          nav = 0
          spin=input_data(0).spin
          for j=1L,n_elements(time)-1 do begin
	      if input_data(j).spin eq spin then begin
                 tps(nav)=tps(nav)+time(j)
                 np=np+1
              endif else begin
	         if np ne 0 then tps(nav)=tps(nav)/float(np) $
	         else tps(nav)=0.d0
	         nav=nav+1
                 tps(nav)=time(j)
       	         np=1
	         spin=input_data(j).spin
              endelse
          endfor
          if np ne 0 then tps(nav)=tps(nav)/float(np) $
          else tps(nav)=0.d0
       endif



       index=where(tps ne 0)
       time=tps(index)
       yray=yray(index,*)


       yst=yminarr(0)
       yen=ymaxarr(n_elements(ymaxarr)-1)


;
; Priority Channels
;
    ENDIF ELSE BEGIN
        ;

       ;If user selected priority channels 4 or 5 for time when only had 0-3 then turn them off
       IF currindex LT 2 THEN BEGIN
            FOR n=0,n_elements(res)-1 DO BEGIN
                IF res[n] GT 3 THEN BEGIN
                    res[n] = 999
                ENDIF
            ENDFOR
            at = WHERE(res[*] NE 999)
            IF at[0] NE -1 THEN res = res[at]
            utitle = strarr(n_elements(res))
       ENDIF
       yray=fltarr(count_prior,n_elements(yminarr))
       tps=dblarr(count_prior)
       IF type(1) EQ 0 THEN uztit='[counts]' $
       ELSE uztit='[counts/acc]'
       uytitle='subsector'
       utitle='CASSINI-MIMI/LEMMS!c'+currprior(prior_index)

       if type(0) eq 0 then utitle=utitle+' not av.'
       if type(0) eq 1 then utitle=utitle+' spin av.'

       nodat=4294967295ll



       for i=0,15 do begin
          indix=where(input_data_prior.subsector eq i, count_prior)
          input_data_prior_buf=input_data_prior(indix)
          if type(0) eq 0 then begin  ;no average
             yray(0:count_prior-1,i)=input_data_prior_buf(*).data(prior_index)
          endif

          if type(0) eq 1 then begin  ;spin average
	     if input_data_prior_buf(0).data(prior_index) ne nodat then begin
                yray(0,i)=input_data_prior_buf(0).data(prior_index)
       	        np=1
	     endif else begin
	        yray(0,i)=0.
	        np=0
	     endelse
	     nav = 0
	     spin=input_data_prior_buf(0).spin
             for j=1L,n_elements(input_data_prior_buf)-1 do begin
	        if input_data_prior_buf(j).spin eq spin then begin
	          if input_data_prior_buf(j).data(prior_index) ne nodat then begin
	             yray(nav,i)=yray(nav,i)+input_data_prior_buf(j).data(prior_index)
                     np=np+1
	          endif
                endif else begin
	          if np ne 0 then begin
                     yray(nav,i)=yray(nav,i)/float(np)
	          endif else begin
	             yray(nav,i)=nodat
	          endelse
	          nav=nav+1
	          if input_data_prior_buf(j).data(prior_index) ne nodat then begin
                     yray(nav,i)=input_data_prior_buf(j).data(prior_index)
       	             np=1
	          endif else begin
	             yray(nav,i)=0.
	             np=0
	          endelse
	          spin=input_data_prior_buf(j).spin
                endelse
	     endfor
	     if np ne 0 then begin
                yray(nav,i)=yray(nav,i)/float(np)
	     endif else begin
	        yray(nav,i)=nodat
	     endelse
          endif
       endfor

       if type(0) eq 0 then begin  ;average time_prior over subsectors
          tps(0)=time_prior(0)
          np=1
          nav = 0
          sect=input_data_prior(0).sector
          for j=1L,n_elements(time_prior)-1 do begin
	      if input_data_prior(j).sector eq sect then begin
                 tps(nav)=tps(nav)+time_prior(j)
                 np=np+1
              endif else begin
	         if np ne 0 then tps(nav)=tps(nav)/float(np) $
	         else tps(nav)=0.d0
	         nav=nav+1
                 tps(nav)=time_prior(j)
       	         np=1
	         sect=input_data_prior(j).sector
              endelse
          endfor
          if np ne 0 then tps(nav)=tps(nav)/float(np) $
          else tps(nav)=0.d0
       endif

       if type(0) eq 1 then begin  ;average time_prior over spin
          tps(0)=time_prior(0)
          np=1
          nav = 0
          spin=input_data_prior(0).spin
          for j=1L,n_elements(time_prior)-1 do begin
	      if input_data_prior(j).spin eq spin then begin
                 tps(nav)=tps(nav)+time_prior(j)
                 np=np+1
              endif else begin
	         if np ne 0 then tps(nav)=tps(nav)/float(np) $
	         else tps(nav)=0.d0
	         nav=nav+1
                 tps(nav)=time_prior(j)
       	         np=1
	         spin=input_data_prior(j).spin
              endelse
          endfor
          if np ne 0 then tps(nav)=tps(nav)/float(np) $
          else tps(nav)=0.d0
       endif

       index=where(tps ne 0)
       time_prior=tps(index)
       yray=yray(index,*)

       yst=yminarr(0)
       yen=ymaxarr(n_elements(ymaxarr)-1)


    ENDELSE   
; the variables used above have the following meaning:
;       uztit   : the z-axis (papco_colorbar) label
; 	uytitle : the y-axis label
;	utitle  : the right-of plot label defining the data




; the procedure autorange is optional and sets the zlimit automatically for
; the data, attempting to use the full dynamic range of the color table.
; you may need to supply your own procedure for your own data. 
; (in $papco/PLOT_prog/Common)

; yscl or zscale is an array of dimensions (*,4) and contains info
; on the scaling to be used. The first dimension identifies the panel, and the
; next dimension has three entries:  
;         yscl(*,0)  is  0 for "use automatic scaling", 1 for "use manual"
;         yscl(*,1)  is then ymin (or zmin)
;         yscl(*,2)  is then ymax (or zmax)
;         yscl(*,3)  is 0 for lin, 1 for log axis
; when scaling is set to automatic, the automatic y limits are loaded in.
; for color plots, the z-limits are used for the color (z-axis) scaling
   
   if (zscl(panel(0),0) eq 1) then begin
      zmin=zscl(panel(0),1)
      zmax=zscl(panel(0),2)
   endif else begin
;      autorange,panel(0),time,yray,zmax,zmin,1
      papco_autorange,yray,zmin,zmax,LOG=yscl(panel(0),3),nodat=nodat,exclude=5.,/verbose
      zscl(panel(0),1)=zmin
      zscl(panel(0),2)=zmax
   endelse
   if zscl(panel(0),3) eq 1 and ABS(floor(ALOG10(zmax))-floor(Alog10(zmin))) lt 1 then zmin=zmin/10.

   if (yscl(panel(0),0) eq 1) then begin
      yst=yscl(panel(0),1)
      yen=yscl(panel(0),2)
   endif else begin
      yscl(panel(0),1)=yst
      yscl(panel(0),2)=yen
   endelse

; uae auto y-labeling routine to get "good" y-lables. This sets the global IDL
; graphics system variables !Y.MINOR, !Y.TICKV, !Y.TICKS - so if you want to 
; use this feature, avoid setting these or their keywords in your plot call.
  papco_y_label, yst, yen, log=yscl(panel(0),3)    


; set up extra plot keywords, common ones first. 
; yrange, xrange  are always needed. 
; the rest you can set if you want to override papco defaults.
; See $papco_BASE/papco/papco_draw for the definition of the papco default
; graphic sytem variable settings.
  extra_plotPar_common={yrange:[yst,yen], xrange:[xut1,xut2], zrange:[zmin,zmax], $
                 ylog:yscl(panel(0),3), zlog:zscl(panel(0),3), $
                 ztitle:uztit, ystyle:1, $
                 xtickformat:'noticks',ztickformat:'papco_color_bar_log_ticks'}


; add keyword structure set here with the one passed in

   extra_plotPar=create_struct(extra_plotPar_common, extra_par)


; use papco routine to draw time axis. This checks for bottom plot and uses the
; user's xtickformat if it is something other than 'noticks'.
; If you use this rotuine, make sure your data plot routine uses xstyle=5 and 
; ystyle=5 (inhibit axis) so that you don't clobber those drawn by papco.  
; "down" sets the axis labeling level. 
  down=0
  papco_draw_time_axis, panel, OVERPLOT=OVERPLOT, _extra=extra_plotPar


; do color plot and color bar plot. nodata specifies the "nodata" flag
; in your data. Here we do not plot axis and data seperate, as
; papco_plot_colorspec cannot be used as an overplot!
  
  ;check the routine papco_plot_colorspec in papco_XX\papco_lib\color_plot
  ;for all the possible options of papco_plot_colorspec.
  ;Hint: you might want to use the resx, resy keywords, which set the
  ;resolution to plot add. Without these, the routine will try to use
  ;device resolution, which for postscript may result in HUGE files!
  ;resx, resy are in data coordinates. If the axis is logarithmic, 
  ;resx, resy need to be too.
  
  
  IF channel LT 60 THEN  resx=(time(n_elements(time)-1)-time(0))/n_elements(time)/2. $
  ELSE resx=(time_prior(n_elements(time_prior)-1)-time_prior(0))/n_elements(time_prior)/2.
  
  if yen lt yst then begin
     index=where(yminarr ge yen and yminarr le yst,n_channels)
  endif else begin
     index=where(yminarr ge yst and yminarr le yen,n_channels)
  endelse
  n_channels=n_channels+1
  
  if (yscl(panel(0),3) eq 0) then begin
    resy=ABS((yen-yst)/n_channels/2.)
  endif else begin
    resy=ABS((ALOG(yen)-ALOG(yst))/n_channels/2.)
  endelse
  
  
  IF channel LT 60 THEN  $
    papco_plot_colorspec,yray,time,yarr, nodata=nodat, resx=resx, resy=resy, $
        xstyle=5, ystyle=5, _extra=extra_plotPar $
  ELSE papco_plot_colorspec,yray,time_prior,yarr, nodata=nodat, resx=resx, resy=resy, $
        xstyle=5, ystyle=5, _extra=extra_plotPar
        
  papco_color_bar, _extra=extra_plotPar
                    
; plot y-axis label at left of plot. Use scalable routine!  
  left_side_label,panel,uytitle,/rot90  
  
; store the coordinate information into common block
  plotted_x = !x  &  plotted_y = !y

; Since here papco_colorbars are plotted, labels are rotated by 90
; degrees and then plotted. No allowance is made for overplots, as
; color plots cannot be an overplot.
  right_side_label,panel,utitle,/rot90

end 







