;****************************************************************************
;*  NAME:        r_cassini_mimi_lemms
;*
;*  DESCRIPTION: Routine to read cassini_mimi_lemms files. Adheres to
;*               PAPCO conventions: Error common block
;*                                  Uses MJDT time
;*                                  PAPCO read philosophy
;*
;*  INPUT:       none (uses common block time from PAPCO)
;*
;*  OUTPUT:      none (places data into common blocks)
;
;*  Notes:      Lemms priority channels
;               before 1999-229T11:28 channels A0 A1 E0 E1
;               after 1999-229T11:28 channels A0 A1 C0 C1
;               after 2003-255T02:39 channels       A1 A5 C1 C5 E4 P2
;*
;* MODIFICATION HISTORY:
;*     written April 2001, S. BOURDARIE (ONERA/DESP)
;   3/18/2002 M.B. Kusterer Changed the routine to read the one day files. Stopped hourly loop
;   3/20/2002 M.B. Kusterer Changed the routine to read in all the priority channels, also deal
;                           with very low telemetry mode.
;   Sept. 4 2001, M.B.Kusterer  Added call to papco_get_remote_data to r_cassini_mimi_lemms. Also
;                           Added 2 more priority channels and full 60 normal channels to anticipate
;                           a coming flight sw change. Added telRate also.
;                           This way I won't change the IDL save files format.
;   April. 7 2003, M.B.Kusterer  Changed to have the priority data in a separate structure. Also changed
;                           path initialization to not contain the year. The last year directory is done by
;                           the file title calculation. Will eliminate having to change path for each year.
;                           I also added min and max of A0 and C0 to the input_data to speed up light removal
;                           process.
;   Mar 1 2004, M.B.Kusterer  Changed to work with the changed and 2 additional priority channels. Also changed
;									the directory for the ftp data to be papdata_v35.
;               For light correction in data before 1999-229 use A0, A1, for data after 2003-255 use A1 C1 
;****************************************************************************
pro r_cassini_mimi_lemms, VERBOSE=VERBOSE

   common cassini_mimi_lemms_data, input_header, input_data, input_data_prior
   common mjdt, mjdt_start, mjdt_end
   common get_error, get_err_no, get_err_msg

; common get_error is used by PAPCO to check if the data read was successful.
; it is up to the user to set this correctly and to make sure that the read-
; routine does not crash on an I/O error - otherwise PAPCO will crash at that
; point, too. PAPCO draws an empty panel when get_err_no in not equal to zero
; and prints get_err_msg in that panel for user information. Papco will try
; to invoce the plot routine for this data if get_err_no = 0 , otherwise an
; empty panel with the error message get_err_msg is plotted.

   get_err_no=0
   get_err_msg='cassini_mimi_lemms read succesfully'

; check for env. variable pointing to data directory
   data_dir='CASSINI_MIMI_LEMMS_DATA'


   c_path=getenv(data_dir)
   if strlen(c_path) eq 0 then begin
      get_err_no=1
      get_err_msg='Environmental variable '+data_dir+' not set'
      print,get_err_msg
      return
   endif

; check for existence of data directory
   ; assemble the data file from the time range requested
   time=mjdt_start
   convert_t90_to_date, time, year, day_oy, hour, minute, second,$
                          month, cmonth, day_om, /mjdt
   yy = string(year,format='(i4.4)')
   res=findfile(c_path+'/'+yy+'/',count=cc)
   if cc eq 0 then begin
      get_err_no=2
      get_err_msg='Data directory '+c_path+' not found'
      print,get_err_msg
     return
   endif


; check VERBOSE keyword and set
   if keyword_set(VERBOSE) then verbose=VERBOSE else verbose=0

; clear data arrays for reuse
   input_header=0  &  input_data=0  & input_data_prior=0

; loop through from start to end time concatonating data files into one
; array. Here we assume that the data is stored in files per day, and
; that the time info in each file is seconds of day. papco standard is
; to hand over data to the plot routines with the time running in
; seconds from the number of seconds of the startday.

   time=mjdt_start  &  no_files_read=0  &  day_count=0.0
        ;MBK
   oldfile = ''
        ;MBK
   while mjdt_end.mjd ge time.mjd do begin
     if mjdt_end.mjd eq time.mjd then hour_end = mjdt_end.t else hour_end=86400.
     while time.t le hour_end do begin

        ; assemble the data file from the time range requested
        convert_t90_to_date, time, year, day_oy, hour, minute, second,$
                          month, cmonth, day_om, /mjdt
        ;MBK
        fln = '/'+string(year,format='(i4.4)')+'/cmilea_1A_' $
                 +string(year,day_oy,format='(i4.4,i3.3)')
        newfile = fln


        IF newfile NE oldfile THEN BEGIN

                found=findfile(c_path+fln+'*.*',count=nfs)  ;are files there *.idl or *.txt or .txt.gz?
                if nfs ne 0 then begin
                    ; read one hour of data
                    resul = str_sep(found(0),'.')
                    r_cassini_mimi_lemms_onefile, resul(0), /VERBOSE


                    ;if a file was succesfully read, concatonate!
                    if get_err_no eq 0 then begin
                        if no_files_read eq 0 then begin   ;start the array buffer
                            input_data_buf=temporary(input_data)
                            input_data_prior_buf=temporary(input_data_prior)
                            input_header_buf = input_header
                        endif else begin                   ;otherwise concatonate arrays
                            input_data_buf=[temporary(input_data_buf),temporary(input_data)]
                            input_data_prior_buf=[temporary(input_data_prior_buf),temporary(input_data_prior)]
                        endelse
                        input_data=0                     ;clear array for reuse
                        no_files_read=no_files_read+1
                    endif

               endif else begin    ;didn't fine any files to match, try ftp from mimi
                    tempname = c_path+fln
                    print,'% failure to find: '+tempname+' trying ftp next ...'

                    ;make remote path fln is local path/file with no extension
                    ll = STRLEN(tempname)
                    year = STRMID(tempname,ll-7,4)
                    file = STRMID(tempname,ll-17,26)      ;filename with no path
                    ldir = STRMID(tempname,0,ll-17)    ;path to place file at locally
                    rdir = 'papdata_v35/lemmsrates/'+year+'/' ;remote directory to find file

                    ;call get remote data
                    papco_get_remote_data,'ftp://mimipap:MIMIpap1@sd-ftp.jhuapl.edu//'+rdir,rdir+file+'*.idl',ldir

                    found=findfile(c_path+fln+'*.*',count=nfs)
                    if nfs ne 0 then begin
                        resul = str_sep(found(0),'.')

                        r_cassini_mimi_lemms_onefile, resul(0), /VERBOSE
                        ;if a file was succesfully read, concatonate!
                        if get_err_no eq 0 then begin
                            if no_files_read eq 0 then begin   ;start the array buffer
                                input_data_buf=temporary(input_data)
                                input_data_prior_buf=temporary(input_data_prior)
                                input_header_buf = input_header
                            endif else begin                   ;otherwise concatonate arrays
                                input_data_buf=[temporary(input_data_buf),temporary(input_data)]
                                input_data_prior_buf=[temporary(input_data_prior_buf),temporary(input_data_prior)]
                            endelse
                            input_data=0                     ;clear array for reuse
                            no_files_read=no_files_read+1
                        endif
                    endif   ;2nd try at nfs ne 0


                endelse     ;nfs ne 0
                oldfile = newfile
            ENDIF ELSE print,'files match'  ;newfile ne oldfile
            ;MBK
	        time.t = time.t+(3600.*24.)
      endwhile
      time.t = 0.
      time.mjd=time.mjd+1
   endwhile

; check if any files were read
   if (no_files_read ne 0) and (get_err_no ne 1) then begin
     get_err_no=0
     get_err_msg='% R_CASSINI_MIMI_LEMMS: '+string(no_files_read,format="(i2)")+ $
       ' files read succesfully'
     ;clear arrays and load buffer into output array
     input_data=0  &  input_data=input_data_buf  &  input_data_buf=0
     input_data_prior=0  &  input_data_prior=input_data_prior_buf  &  input_data_prior_buf=0
     input_header = input_header_buf

;converting time to PAPCO plot standard

     input_data(*).time=(input_data(*).mjdt_time.mjd-mjdt_start.mjd) $
                     *86400+input_data(*).mjdt_time.t
     input_data_prior(*).time=(input_data_prior(*).mjdt_time.mjd-mjdt_start.mjd) $
                     *86400+input_data_prior(*).mjdt_time.t
   endif else begin

     get_err_no=1
     get_err_msg='% R_CASSINI_MIMI_LEMMS: no files found for timerange'
     return
   endelse

; restrict data to actual time range requested
   ut1=mjdt_start.t
   ut2=long(mjdt_end.mjd-mjdt_start.mjd)*86400l+mjdt_end.t
   index=where((input_data.Time ge ut1) and (input_data.Time le ut2),count)

   if count ne 0 then begin
      input_data=input_data(index)
      index_prior = index[0]*8 + L64INDGEN(count*8)
      input_data_prior=input_data_prior(index_prior)
   endif else begin
     get_err_no=1
     get_err_msg='% r_cassini_lemms: no data found for timerange'
     return
   endelse

end

;******************************************************************************
;*  NAME:        r_cassini_mimi_lemms_onefile,fln
;*
;*  DESCRIPTION: reads one file of type plot_file. Assumes data has been
;*               stored in IDL saveset using structures input_header and
;*               input_data
;*
;*  INPUT:       fln  - string of filename
;*
;*  OUTPUT:      none (places data into common blocks)
;*
;* MODIFICATION HISTORY:
;*     written April 2001, S. BOURDARIE (ONERA/DESP)
;******************************************************************************
pro r_cassini_mimi_lemms_onefile, fln, VERBOSE=VERBOSE

   common cassini_mimi_lemms_data, input_header, input_data, input_data_prior
   common get_error, get_err_no, get_err_msg
   common mjdt, mjdt_start, mjdt_end    ;common time limit in mjdt
   
   get_err_no=0
   get_err_msg='cassini_mimi_lemms onefile read succesfully'

; clear data arrays for reuse
   input_header=0  &  input_data=0

   on_ioerror, read_original
   restore,fln+'.idl',/verbose


    ;zero out unused prioirty channels
   ;nodat=4294967295ll
    ;figure out priority channel name based on time
    ;cassini_mimi_lemms_priority, mjdt_start, currprior, numprior, currindex
   ; IF currindex LT 2 THEN BEGIN 
    ;mn = MIN(input_data_prior(*).data[4:5],MAX=mx)
    ;print,'min ',mn,' max ',mx
    ;        input_data_prior(*).data[4:5] = nodat
    ;ENDIF
    

   dealloc_lun,/quiet    ;frees logical units used by save/restore
   print,'% R_CASSINI_MIMI_LEMMS_ONEFILE: idl saveset read succesfully'
   return


   read_original:
   print,'% R_CASSINI_MIMI_LEMMS_ONEFILE: failure to find: '+fln+'.idl trying .txt file ...'
   ; if the .idl save file could not be found, then try to read the original
   r_cassini_mimi_lemms_oneday_original, fln
   return

end

;****************************************************************************
;*  NAME:        r_cassini_mimi_lemms_oneday_original
;*
;*  DESCRIPTION: Routine to read cassini_mimi_lemms files. Adheres to
;*               PAPCO conventions: Error common block
;*                                  Uses MJDT time
;*                                  PAPCO read philosophy
;*
;*  INPUT:       none (uses common block time from PAPCO)
;*
;*  OUTPUT:      none (places data into common blocks)
;*
;* MODIFICATION HISTORY:
;*     written April 2001, S. BOURDARIE (ONERA/DESP)
;   3/20/2002 M.B. Kusterer Changed the routine to read in all the priority channels, also deal
;                           with very low telemetry mode. Had to add code to find telemetry rate
;   Sept. 4 2001, M.B.Kusterer  Added call to papco_get_remote_data
;****************************************************************************
pro   r_cassini_mimi_lemms_oneday_original, fln

   common cassini_mimi_lemms_data, input_header, input_data, input_data_prior
   common get_error, get_err_no, get_err_msg

;  clear exisiting data array
   input_data=0

;before 1999-229T11:28 channels A0 A1 E0 E1
;after 1999-229T11:28 channels A0 A1 C0 C1
;after 2003-255 channels A1 A5 C1 C5 E4 P2

   time={MJDT, MJD:0l, T:0.0}
   dummystr=''
   dummytime=' 0 0 0'
   num_prior = 6    ;number of priority channels
   num_norm = 60    ;number of normal channels

    ; structure for normal channels
   dummy_struct={    data      :lon64arr(num_norm), $
                    A0_minmax:lon64arr(2),$
                    C0_minmax:lon64arr(2),$
                     spin      :0, $
                     sector    :0, $
		            subsector :0, $
		            spinPeriod:0L, $
                    telRate:0,$
                     mjdt_time :time, $
                     time      :0.0d0}

    ;additional structure for priority channels
   dummy_struct2={    data      :lon64arr(num_prior), $
                    A0:LONG64(0),$
                     spin      :0, $
                     sector    :0, $
		            subsector :0, $
		            microsector :0, $
		            spinPeriod:0L, $
                    telRate:0,$
                     mjdt_time :time, $
                     time      :0.0d0}

   input_data=replicate(dummy_struct,33550)    ;figure on 11 minute spin fastest ever allowed
                                                ; so 256 per spin * 131 spins per day
   input_data_prior=replicate(dummy_struct2,268300)    ; so 256 per spin * 131 spins per day * 8 microsectors

   openr,f_Unit,fln+'.txt', /GET_LUN, ERROR=err

   IF err NE 0 THEN BEGIN
      openr,f_Unit,fln+'.txt.gz',/GET_LUN,/compress, ERROR=err
      IF err ne 0 THEN BEGIN
         print, '% READ_CASSINI_MIMI_LEMMS: '+fln+'.txt'+'  :  not available.'
         get_err_no=2
         get_err_msg=!ERR_STRING
         print, !ERR_STRING
         RETURN
       ENDIF
   ENDIF

   print,'% R_CASSINI_MIMI_LEMMS_ASCII: Reading ',fln+'.txt',' (slow)'
; read the header
   burp = ''
   readf,f_Unit,burp
   while (strmid(burp,0,12) ne 'Spin_Counter') do readf,f_Unit,burp

   strdata=''
   ;data = lon64arr(129)
   data = lon64arr(162)
   idata= lonarr(6)

   i=-1ll ;i is index into input_data new array
        ;j is index into data, new for each read
   p=-1ll ;p is index into input_data_prior new array, counts either 8 or 16 per line in science record


   nodat=4294967295ll
   input_data(*).data[*] = nodat   ;blank
   input_data_prior(*).data[*] = nodat   ;blank
   mstimes=REPLICATE({MJDT},16)

   beg_prio_i = num_norm+num_prior
   end_prio_i = (num_norm+num_prior-1)+num_prior
   

   while not eof(f_Unit) do begin
      i=i+1 ;i counts the lines in the science records, matches # normal counters
      if i GT N_ELEMENTS(input_data) then print,$
      '<<< WARNING >>>> Outside bounds of normal array in r_cassini_mimi_lemms'
      if p GT N_ELEMENTS(input_data_prior) then print,$
      '<<< WARNING >>>> Outside bounds of priority array in r_cassini_mimi_lemms'

      readf,f_Unit,strdata
      year = fix(strmid(strdata,176,4))
      doy  = fix(strmid(strdata,181,3))
      hour = fix(strmid(strdata,185,2))
      minutes = fix(strmid(strdata,188,2))
      seconds = float(strmid(strdata,191,6))
      tel_rate =    fix(strmid(strdata,220,2))
      ;input_data(i).spinPeriod=long(strmid(strdata,110,10))


      input_data(i).mjdt_time=convert_date_to_t90(doy=doy, year=year,$
                               hour=hour,minutes=minutes,seconds=seconds, /MJDT)
      reads,strdata,idata
      input_data(i).spin     =idata(0)
      input_data(i).sector   =idata(1)
      input_data(i).subsector=idata(2)
      input_data(i).spinPeriod=idata(5)
      input_data(i).telRate=tel_rate

      str = strmid(strdata,241,strlen(strdata)-241)

      ;reads,str,data,format='(129(I22))'
      reads,str,data,format='(162(I22))'
      

      input_data(i).data[0:num_norm-1]  = data(0:num_norm-1)    ;last 3 normal channels are blank
      input_data(i).data[num_norm-3:num_norm-1]  = nodat    ;last 3 normal channels are blank


      ;find the seconds to add for the 8 microsectors
      mssecs = (FLOAT(input_data(i).spinPeriod)*64.*16./(6.*1000000.)/8.0)*FINDGEN(16)

      mstimes[*].MJD = input_data(i).mjdt_time.MJD
      mstimes[*].T = input_data(i).mjdt_time.T
 ;
        ;figure out priority channel name based on time
        ;
       flsw_1={MJDT, MJD:0l, T:0.0}
       flsw_1=convert_date_to_t90(doy=229, year=1999,$
                               hour=11,minutes=28,seconds=0, /MJDT)
       flsw_1_all = float(flsw_1.mjd)+(flsw_1.t/(24.0*60.0*60.0))
        
       flsw_2={MJDT, MJD:0l, T:0.0}
       flsw_2=convert_date_to_t90(doy=255, year=2003,$
                               hour=2,minutes=39,seconds=0, /MJDT)
       flsw_2_all = float(flsw_2.mjd)+(flsw_2.t/(24.0*60.0*60.0))
        
       curtime_all = float(input_data(i).mjdt_time.MJD)+(input_data(i).mjdt_time.T/(24.0*60.0*60.0))
        
      ;
      ;Load min and max of A0 and C0 in input_data to speed up light removal routine
      ;
      
       IF curtime_all GE flsw_2_all THEN BEGIN              ;after 2003-255 channels A1 A5 C1 C5 E4 P2
       
      	A0_indexes = (lindgen(8)*num_prior)+beg_prio_i      ;use A1 instead of A0
      	C0_indexes = (lindgen(8)*num_prior)+beg_prio_i+2    ;use C1 instead of C0
      	input_data(i).A0_minmax[*] = [MIN(data(A0_indexes)),MAX(data(A0_indexes))]
      	input_data(i).C0_minmax[*] = [MIN(data(C0_indexes)),MAX(data(C0_indexes))]
        
       ENDIF ELSE IF curtime_all LT flsw_1_all THEN BEGIN   ;before 1999-229T11:28 channels A0 A1 E0 E1
       
      	A0_indexes = (lindgen(8)*num_prior)+beg_prio_i
      	C0_indexes = (lindgen(8)*num_prior)+beg_prio_i+1    ;use A1 instead of C0
      	input_data(i).A0_minmax[*] = [MIN(data(A0_indexes)),MAX(data(A0_indexes))]
      	input_data(i).C0_minmax[*] = [MIN(data(C0_indexes)),MAX(data(C0_indexes))]
        
       ENDIF ELSE BEGIN ;between those two dates            ;after 1999-229T11:28 channels A0 A1 C0 C1
       
      	A0_indexes = (lindgen(8)*num_prior)+beg_prio_i
      	C0_indexes = (lindgen(8)*num_prior)+beg_prio_i+2
      	input_data(i).A0_minmax[*] = [MIN(data(A0_indexes)),MAX(data(A0_indexes))]
      	input_data(i).C0_minmax[*] = [MIN(data(C0_indexes)),MAX(data(C0_indexes))]
        
       ENDELSE 

      ;
      ; priortiy channels
      ;
      ;p=first of set
      p=p+1 ;p counts the lines of priority microsectors in the science records

      input_data_prior(p).data[0:num_prior-1]  = data(beg_prio_i:end_prio_i)
      ; IF curtime_all LT flsw_2_all THEN BEGIN              ;zero out unused priority channels
       ;     input_data_prior(p).data[4:5] = nodat
      ;ENDIF
      input_data_prior(p).mjdt_time = mstimes[0]
      input_data_prior(p).A0     =data(0)
      input_data_prior(p).spin     =idata(0)
      input_data_prior(p).sector   =idata(1)
      input_data_prior(p).subsector=idata(2)
      input_data_prior(p).microsector=0
      input_data_prior(p).spinPeriod=idata(5)
      input_data_prior(p).telRate=tel_rate

      FOR ii=1,7 DO BEGIN
        if (mssecs[ii]+mstimes[ii].T) GT 86400.0 THEN BEGIN
            mstimes[ii].T = mssecs[ii]+mstimes[ii].T-86400.0
            mstimes[ii].MJD = mstimes[ii].MJD+1
        ENDIF ELSE mstimes[ii].T = mssecs[ii]+mstimes[ii].T

        p = p+1
        input_data_prior(p).data[0:num_prior-1]  = data(beg_prio_i+(ii*num_prior):end_prio_i+(ii*num_prior))
        ;IF curtime_all LT flsw_2_all THEN BEGIN              ;zero out unused priority channels
        ;    input_data_prior(p).data[4:5] = nodat
        ;ENDIF
        input_data_prior(p).mjdt_time = mstimes[ii]
        input_data_prior(p).A0     =data(0)
        input_data_prior(p).spin     =idata(0)
        input_data_prior(p).sector   =idata(1)
        input_data_prior(p).subsector=idata(2)
        input_data_prior(p).microsector= (ii*2)
        input_data_prior(p).spinPeriod=idata(5)
        input_data_prior(p).telRate=tel_rate

      ENDFOR

      if (tel_rate EQ 1) then begin ;very low rate has an additional set of priority
                                    ;channels at subsector + 1
       myii = 0
       FOR ii=8,15 DO BEGIN
        if (mssecs[ii]+mstimes[ii].T) GT 86400.0 THEN BEGIN
            mstimes[ii].T = mssecs[ii]+mstimes[ii].T-86400.0
            mstimes[ii].MJD = mstimes[ii].MJD+1
        ENDIF ELSE mstimes[ii].T = mssecs[ii]+mstimes[ii].T

        p = p+1

        input_data_prior(p).data[0:num_prior-1]  = data(beg_prio_i+(ii*num_prior):end_prio_i+(ii*num_prior))
        input_data_prior(p).mjdt_time = mstimes[ii]
        input_data_prior(p).A0     =data(0)
        input_data_prior(p).spin     =idata(0)
        input_data_prior(p).sector   =idata(1)
        input_data_prior(p).subsector=idata(2) +1
        input_data_prior(p).microsector= (myii*2)
        
        input_data_prior(p).spinPeriod=idata(5)
        input_data_prior(p).telRate=tel_rate
        
        myii = myii+1
      ENDFOR

      endif
   endwhile

   close,f_Unit
   free_lun,f_Unit

   IF i le 0 THEN BEGIN
      print, '% readcassini_mimi_lemms: '+fln+'.txt'+'  :  not available.'
      get_err_no=2
      get_err_msg=!ERR_STRING
      print, !ERR_STRING
      RETURN
   ENDIF

   input_data=temporary(input_data(0:i))
   input_data_prior=temporary(input_data_prior(0:i*8))


; save read data with correct time. Rename data to *.idl
  save,filename=fln+'.idl',input_data, input_data_prior,/verbose

END
